package dmoisan.text;
import java.io.*;
/**
* TemplateStream
* 	A class for processing text in a stream against a supplied template so that
*	user-defined keywords within the template are replaced with the user's
*	text (e.g. <NAME> is replaced with "David Moisan").
*    Designed for Scala Lingua scripts but can be used for 
*    general-purpose template loading and replacement.
*
* <p>Permission is granted to use this code in any commercial or non-commercial 
* application.  NO WARRANTY IS IMPLIED.</p>
*
* @author David Moisan
* @version 0.8 22 May 1999
*/
public class TemplateStream {

/*
* Class variables
*/
private static final boolean DEBUG=false;
/*
* Instance variables
*/

private String TemplateContents="";
private String CurrentTemplate="";
private String TemplateName ="";
private String TemplateStartString="";
private String TemplateEndString="";
private String TemplateKeywordStart="";
private String TemplateKeywordEnd="";

private int templatecurpos;

/**
* Constructor
*
* @param contents String Contents of all templates
*/
     public TemplateStream(String contents) {
          this.TemplateContents = contents;
          return;
          }
/*
* Public instance methods
*/

/**
* SetTemplateDelim --
*   Sets the strings that delimit the start and end 
*   of the template, and the command prefix that differentiates
*   substitution keywords from plain text
* 
* @param String start  Opening string in template
* @param String end Closing string in template
* @param String keystart Keyword opening delimiter
* @param String keyend Keyword closing delimiter
*/

     public void setTemplateDelim
          (String start, String end, String keystart, String keyend) {
          this.TemplateStartString = start;
          this.TemplateEndString = end;
          this.TemplateKeywordStart = keystart;
          this.TemplateKeywordEnd = keyend;
     }	
/**
* newTemplate --  Load a new template to match keywords against in future
* processing
*
* Note that name is appended to the template start string 
* and the search is done on that string.  
* 
* @param name String Name of template to load
* @returns boolean true if template loaded, false if not
*/
     public boolean newTemplate(String name) {
          int startdelimpos, enddelimpos;
          String templatestartname = this.TemplateStartString+name;
            if (this.TemplateName.equals(name)) {
               this.templatecurpos = 1; // reset pointer
               return true; //and do NOT reload template
               }
          else {
               startdelimpos = this.TemplateContents.indexOf(templatestartname);
               if (startdelimpos != -1) {
                    enddelimpos = this.TemplateContents.indexOf(this.TemplateEndString, startdelimpos);
                    if (enddelimpos != -1) {
                         this.CurrentTemplate = 
                              this.TemplateContents.substring
                                   (startdelimpos-1, enddelimpos+TemplateEndString.length());
                         this.TemplateName = name;
                         // reset template pointer now that it's loaded
                         // 
                         this.templatecurpos = 1;                         
                         return true;
                         }
                    else {
                         return false; 
                         }
                    }
               else {
                    return false;
                    }
               }
          }    
     

/**
* replaceString
*   The heart of TemplateStream:
*   Copy the template to the output stream, replacing the 
*   keyword with the specified text.
*
* @param keyword String Keyword to replace
* @param replacement String text that replaces keyword
* @param deststream FileWriter Output stream
* @exception IOException Thrown for I/O errors
* @returns boolean true if keyword found and replaced, false if not.
*/ 
     public boolean replaceString(String keyword, String replacement,
                    FileWriter deststream) throws IOException {
          
          String wholekeyword = this.TemplateKeywordStart+
               keyword+this.TemplateKeywordEnd;
          if (DEBUG) System.out.println
               ("TemplateStream: wholekeyword: "+wholekeyword);
          if (DEBUG) System.out.println
                    ("TemplateStream:this.templatecurpos:"
                    +this.templatecurpos);
          if (DEBUG) System.out.println("Replacement: "+replacement);

          int keywordpos = this.CurrentTemplate.
                              indexOf(wholekeyword, this.templatecurpos);
          if (keywordpos==-1) {
               return false; // Keyword NOT found
               }
          else {
               // Write out all text up to the keyword
               deststream.write
                    (this.CurrentTemplate.substring
                         (this.templatecurpos, keywordpos));
               // and replace it with our text

               deststream.write(replacement);
               // and update the pointer
               this.templatecurpos = keywordpos + wholekeyword.length();
               return true; // Keyword FOUND and replaced...
               }
          }

/**
* closeTemplateOutput -- Write out unused template, replacing all
*                        unused keywords and close output stream
*
* @param deststream FileWriter  Output stream
* @exception IOException  Thrown for I/O errors
*/

     public void closeTemplateOutput(FileWriter deststream) throws IOException {

     int temppointer;
     boolean nokeywords=false;

     while (!nokeywords) {
          temppointer=this.CurrentTemplate.indexOf(TemplateKeywordStart, this.templatecurpos);
          if (temppointer == -1) nokeywords = true;
               // No more keywords found
               
          else {
               // Write out template up to the point where a keyword is
               // inserted.
               //
               deststream.write(this.CurrentTemplate.substring
                    (this.templatecurpos, temppointer));
               this.templatecurpos = temppointer+1;
               temppointer = this.CurrentTemplate.indexOf(
                    TemplateKeywordEnd, this.templatecurpos);
               if (temppointer == -1) nokeywords = true;
                    
               else {
                    this.templatecurpos = temppointer+1;
                    }
               }
          }
     
     /* Flush out remaining unused template to output stream
     */
     deststream.write(this.CurrentTemplate.
                         substring(this.templatecurpos,
                         this.CurrentTemplate.length()));
     return;
     }
}

