package dmoisan.xml.tvschedule;
import java.io.*;
import org.xml.sax.*;
import org.xml.sax.helpers.ParserFactory;

/**
*
* TVScheduleHandler -- Reads TVSCHEDULE XML files and parses them for further processing 
*
*    Handles parsing events and main processing for TVSCHEDULE XML
*    files.  Implements DocumentHandler parsing callback methods
*    and dispatches them to TVSchedule-specific processing methods,
*    processing them with the methods specified in the TVScheduleOutputObject 
*    class.  (So far only ScalaScript is defined but there may be
*    other methods in the future.)  
*
*    Uses Microstar's Aelfred SAX implementation.
*    See ScXML for specific details on use.
*
* <p>Permission is granted to use this code for commercial or non-commercial
* purposes.  NO WARRANTY IS IMPLIED.</p>
*
* @author David Moisan
* @version 0.9 21-May-1999
*
* @see ScXML
* @see TVScheduleOutputObject
* @see ScScript
*/

public class TVScheduleHandler extends HandlerBase {

//
// Class variables

private static final boolean DEBUG = false;          

//
// Instance Variables
//

// StringBuffer ChData -- Character string stored during parsing
//                       and passed along to ScalaScript when elements
//                       DATE, TITLE, TIME and DESCRIPTION are
//                       encountered and completed.

private StringBuffer ChData;

//
// TVScheduleOutputObject TVOut
//   This object is passed to TVScheduleHandler by the calling 
//   routine and defines the actions that TVScheduleHandler will 
//   take to produce the desired output.
//
//   Right now, the ScScript object is the only one in this 
//   class, but other objects can be defined--by the abstract methods in 
//   TVScheduleOutputObject--to write to HTML, plain text, RTF, and
//   any other format needed.
//

//
// parserClass
//   Defines the parser API that will handle the TVSchedule file
//   This class uses the Microstar SAX interface
//

private static final String
     parserClass="com.microstar.xml.SAXDriver";


private TVScheduleOutputObject TVOut;     
private InputSource TVScheduleSource;
private ProgramItem Program;
//
// Constructors
//
/**
* TVScheduleHandler()
*
*
* @param inputxml  Input XML file name
* @param TVOut Class &amp; methods to accept parsed output
*
*/  
   public TVScheduleHandler (String inputxml, TVScheduleOutputObject TVOutArg)
          throws IOException {
          FileReader TVScheduleStream;
          this.TVOut = TVOutArg;
          TVScheduleStream = new FileReader(inputxml);
          this.TVScheduleSource = new InputSource(TVScheduleStream);
          this.ChData = new StringBuffer("");
          return;
          }   

/**
* Init()
*   Start parsing the TVSCHEDULE file according to the information
*   provided in the constructor
*
* @exception Exception   
*/

     public void init() throws Exception {

          Parser TVParse = ParserFactory.makeParser(parserClass);
          TVParse.setDocumentHandler(this);
          TVParse.setErrorHandler(this);
          TVParse.setDTDHandler(this);
          TVParse.parse(TVScheduleSource);
          return;
          }
               

/*
* Required SAX parser event methods.  These are dispatched to specific 
* TVSchedule processing methods
*
*/

/**
*
*
* startElement event: 
*
* @param name Element name (String)
* @param attrs attribute list (if any) (AttributeLIst)
* @exception org.xml.sax.SAXException Thrown by parser.  May enclose
* another exception
*
*/     
     public void startElement (String name, AttributeList attrs)
           throws SAXException {
          try {
               this.ChData.setLength(0);
               // Erase character buffer before
               // processing elements

               if (name.equals("TVSCHEDULE")) {
                    this.ParseStartSchedule(attrs);
                    return;
                    }
          
               else if (name.equals("CHANNEL")) {
                    this.ParseStartChannel(attrs);
                    return; 
                    }

               else if (name.equals("BANNER")) {
                    return;              
                    }
          
               else if (name.equals("DAY")) {
                    return;
                    }

               else if (name.equals("DATE")) {
                    return;
                    }
         
               else if (name.equals("HOLIDAY")) {
                    return;
                    }
          
               else if (name.equals("PROGRAMSLOT")) {
                    this.ParseStartProgram(attrs);
                    return;
                    }

               else if (name.equals("TIME")) {
                    return;
                    }

               else if (name.equals("TITLE")) {
                    this.ParseTitleAttrs(attrs);               
                    return;
                    }

               else if (name.equals("DESCRIPTION")) {
                    return;
                    }

               return;
               }
               catch (IOException e) {
                    throw new SAXException("I/O error: "+e.toString(), e); 
                    }
               catch (TVScheduleParseException e) {
                    throw new
                         SAXException("TVScheduleParseError"+e.toString(), e);
                    }
     }
/**
* endElement event -- For now, just prints name of ending tag
*
* @param name Element name (String)
* @exception org.xml.sax.SAXException Thrown by parser.  May enclose
* another exception
*
*/

     
     public void endElement(String name) throws SAXException  {
          try {
          if (name.equals("TVSCHEDULE")) {
               this.ParseEndSchedule();
               return;
               }    
                         
          else if (name.equals("CHANNEL")) {
               this.ParseEndChannel();
               return;   
               }

          else if (name.equals("BANNER")) {
               this.ParseBanner();
               return;
               }

          else if (name.equals("DATE")) {
               this.ParseStartDay();          
               return;
               }

          else if (name.equals("HOLIDAY")) {
               this.ParseHoliday();
               return;
               }

          else if (name.equals("DAY")) {
               this.ParseEndDay();
               return;
               }

          else if (name.equals("TIME")) {
               this.ParseProgramTime();
               return;
               }
           
          else if (name.equals("TITLE")) {
               this.ParseTitle();               
               return;
               }

          else if (name.equals("DESCRIPTION")) {
               this.ParseDescription();
               return;
               }

          else if (name.equals("PROGRAMSLOT")) {
               this.ParseEndProgramSlot();
               return;
               }

          return;
          }
          catch (IOException e) {
               throw new SAXException("I/O error: "+e.toString(), e); 
                    }
          catch (TVScheduleParseException e) {
               throw new SAXException("TVScheduleParseError"+e.toString(), e);
                    }
     }


/**
* characters 
*   Get character data from parser and append it to our
*   string buffer
*
* @param char[] ch Character data array
* @param int start Start index of character data
* @param int length Length of character data
*/

     public void characters(char[] ch, int start, int length) {
          this.ChData.append(ch, start, length);
          if (DEBUG) System.out.println("Chars: "+this.ChData.toString());
          }

/**
* ignorableWhitespace
*   throw away whitespace and do nothing with it
* @param char[] ch Character data
* @param int start start index of whitespace chars
* @param int length length
*/

     public void ignorableWhitespace(char [] ch, int start, int length)
          {
          if (DEBUG) System.out.println
               ("Whitespace: "+this.ChData.toString());

          return;
          }

/**
*
* SAX error handling methods 
* 
*/


/**
* error()
*    Implements SAX ErrorHandler and prints out an error message
*
* @param e SAXParseException
*
*/

     public void error(SAXParseException e) {
          this.printError("SAX Error", e);
          return;
          }

/**
* warning()
*    Implements SAX ErrorHandler and prints out an error message
*
* @param e SAXParseException
*
*/

     public void warning(SAXParseException e) {
          this.printError("SAX Warning",e);
          return;
          }

/**
* fatalError()
*    Implements SAX ErrorHandler and prints out an error message
*
* @param e SAXException
* @exception org.xml.sax.SAXException Thrown by parser.  May enclose
* another exception
*
*/


     public void fatalError(SAXParseException e) throws SAXException {
          this.printError("SAX Fatal Error",e);
          throw new SAXException(e);
          }



          
/**
*
* TVSchedule specific private processing methods
*/

/**
* printError 
*    Display error messages from SAX;
*
*/
     private void printError(String message, SAXParseException e) {
          int line = e.getLineNumber();
          int col = e.getColumnNumber();
          System.err.println(message+": "+e.getMessage());
          System.err.println("Line: "+line+", Column: "+col);
          return;
          }


//
// ParseStartSchedule(AttributeList attrs)
//   Notify output object of schedule start
//
     private void ParseStartSchedule(AttributeList attrs) throws
          TVScheduleParseException, IOException {
           
          if (DEBUG) System.out.println("ParseStartSchedule()");
          TVOut.newSchedule(attrs.getValue("NAME"));
          }
//
// ParseStartChannel
//   Notify output object of channel start
//
     private void ParseStartChannel(AttributeList attrs)
          throws TVScheduleParseException, IOException {
          if (DEBUG) System.out.println("ParseStartChannel()");
          TVOut.newChannel(attrs.getValue("CHAN"));
          return;
          }
//
// ParseBanner()
//   Send banner to output object
//
     private void ParseBanner() throws TVScheduleParseException, IOException { 
          if (DEBUG) System.out.println("ParseBanner()");
          TVOut.setBanner(this.ChData.toString());
          this.ChData.setLength(0);
          return;
          }

//
// ParseStartDay() -- Signal start of day to output object
//
//
     private void ParseStartDay() throws TVScheduleParseException, IOException {
          if (DEBUG) System.out.println("ParseStartDay(): "+this.ChData.toString());
          TVOut.startDay(this.ChData.toString());
          this.ChData.setLength(0);
          return;
          }

//
// ParseHoliday() 
//   Send Holiday text to output object
//

     private void ParseHoliday() throws TVScheduleParseException, IOException {
          if (DEBUG) System.out.println("ParseHoliday(): "+this.ChData.toString());
          TVOut.setHoliday(this.ChData.toString());
          this.ChData.setLength(0);
          return;
          }

//
// ParseStartProgram
//   Initialize ProgramSlot object
//
     private void ParseStartProgram(AttributeList attrs) {
          if (DEBUG) System.out.println("ParseStartProgram()");
          this.Program = new ProgramItem();
          this.Program.VCR=attrs.getValue("VCR");
          }


// ParseProgramTime
//   Send program time to output object
//
     private void ParseProgramTime() {
          if (DEBUG) System.out.println("ParseProgramTime(): "+this.ChData.toString());
          this.Program.Showtime = this.ChData.toString();
          this.ChData.setLength(0);
          }

//
// ParseTitleAttrs 
//   Parse TITLE attributes
//

     private void ParseTitleAttrs(AttributeList attrs) {
          if (DEBUG) System.out.println("ParseTitleAttrs()");
          this.Program.Rating = attrs.getValue("RATING");
          this.Program.Language = attrs.getValue("LANGUAGE");
          
          if ((attrs.getValue("LIVE") != null) &&
               attrs.getValue("LIVE").equals("YES")) {
               this.Program.Liveshow = true;
               }
          else {
               this.Program.Liveshow = false;
               }

          if ((attrs.getValue("NEW") != null) &&
               attrs.getValue("NEW").equals("YES")) {
               this.Program.Newshow = true;
               }
          else {
               this.Program.Newshow = false;
               }
          }

//
// ParseTitle
//   Send program title to output object
//

     private void ParseTitle()  {
          if (DEBUG) System.out.println("ParseTitle(): "+ this.ChData.toString());
          this.Program.Title = this.ChData.toString();
          ChData.setLength(0);
          }

//
// ParseDescription
//   Send program description to output object
//
     private void ParseDescription() {
          if (DEBUG) System.out.println("ParseDescription(): "+this.ChData.toString());
          this.Program.Description = this.ChData.toString();
          this.ChData.setLength(0);
          return;
          }

  
//
// ParseEndProgramSlot()
//   Send completed program info to the output object
//
     private void ParseEndProgramSlot() throws TVScheduleParseException, IOException {
          if (DEBUG) System.out.println("ParseEndProgramSlot()");
          TVOut.newProgram(Program);
          Program = null;  // Erase Program object when done
          }

         
//
// ParseEndDay()
//   Signal end of day processing
//
     private void ParseEndDay() throws TVScheduleParseException, IOException {
          TVOut.endDay();
          return;                   
          }

//
// ParseEndChannel()
//
     private void ParseEndChannel() throws TVScheduleParseException,  IOException {
          TVOut.endChannel();
          return;
          }
//
// ParseEndSchedule()
//   Signal end of schedule processing
//
     private void ParseEndSchedule() throws TVScheduleParseException, IOException {
          TVOut.endSchedule();
          return;          
          }
}

